<?php

namespace Service;

use Doctrine\ORM\EntityManager;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories;
use Util\Utilities;
use Util\InstanceProvider;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class Dashboard
{
    /** @var \Util\Utilities $environment */
    protected $environment;
    /** @var Repositories\Task $taskRepository */
	protected $taskRepository;
	/** @var Repositories\Resource $resourceRepository */
	protected $resourceRepository;
	/** @var \Service\Task $taskService */
	protected $taskService;

    public function __construct(EntityManager $entityManager, \Util\Utilities $environment, \Service\Task $taskService)
    {
        $this->entityManager      = $entityManager;
        $this->environment        = $environment;
        $this->taskService        = $taskService;
        $this->taskRepository     = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Task');
        $this->resourceRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Resource');
    }

    public function getHoursDeliveredInTheMonthByCurrentUser()
    {
        $currentUserId = $this->environment->getCurrentUserId();
        $arrayFirstAndLastDayCurrentMonth = Utilities::getBeginAndEndDateByCurrentMonthInArray();
        return $this->taskRepository->getHoursDeliveredInTheMonthByCurrentUser($currentUserId, $arrayFirstAndLastDayCurrentMonth[0], $arrayFirstAndLastDayCurrentMonth[1]);
    }
    
    public function getPlannedHoursByCurrentUser()
    {
        $currentUserId = $this->environment->getCurrentUserId();
        return $this->taskRepository->getPlannedHoursByCurrentUser($currentUserId);
    }
    
    public function getTotalIssuesWithPendingConclusion()
    {
        return $this->taskRepository->getTotalIssuesWithPendingConclusion($this->environment->getCurrentOrganizationId(), $this->environment->getCurrentUserId())[0];
    }
    
    public function getTotalDelayedTasksByCurrentUser()
    {
        return $this->taskRepository->getTotalDelayedTasksByCurrentUser($this->environment->getCurrentUserId());
    }
    
    public function getTotalPendingTasksByCurrentUser()
    {
        return $this->taskRepository->getTotalPendingTasksByCurrentUser($this->environment->getCurrentUserId());
    }

    public function getTotalBackLogIssuesByCurrentUser()
    {
        return $this->taskRepository->getTotalBackLogIssuesByCurrentUser($this->environment->getCurrentUserId());
    }
    
    public function getTotalDaysWithAppropriationDifference()
    {
        $totalDifferenceAppropriation = array(
                'FIRST_DAY' => null,
                'TOTAL' => 0
        );

        $user       = $this->environment->getCurrentUser();
        $now        = new \DateTime();
        $yesterday  = $now->modify('-1 day');
        $now        = new \DateTime();
        $last30Day  = $now->modify('-31 day');
        if($user->getAvailableFrom() && $user->getAvailableFrom() > $last30Day ){
            $last30Day = clone $user->getAvailableFrom();
        }
        $allDiffAppropriation = $this->taskService->getDiffAppropriationOfWorkDay($last30Day,$yesterday);

        foreach ($allDiffAppropriation as $diff) {
            if ($diff['HOURS_CONTROL'] != '0') {
                $totalDifferenceAppropriation['TOTAL'] += 1;
                if($totalDifferenceAppropriation['FIRST_DAY'] == null ||
                   (\DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE, $totalDifferenceAppropriation['FIRST_DAY'])) >
                   (\DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE, $diff['DATE']))){
                    $totalDifferenceAppropriation['FIRST_DAY'] =  $diff['DATE'];
                }
            }
        }
        return $totalDifferenceAppropriation;
    }

    /******************************************
    ************* Burndown Methods ************
    *******************************************/

    /* Zeedhi\DevOps\Model\Projectmanagement\Entities\Project */
    public function findOpenSprintByUser()
    {
        $resourceId = InstanceManager::getInstance()->getService('session')->get('userId');
        $addDayInDate = $this->environment->addDayInDate;
        return InstanceProvider::getRepository()->getProject()->findOpenSprintByUser($resourceId, $addDayInDate);
    }

    /** Rotas para o zmart */
    public function findbuildWidgetByViewIdInZmart($drillDownFilters)
    {
        $this->environment->removeTokenZmartInSession();
        $token = $this->environment->getTokenInZmart();
        $accessProfile = InstanceManager::getInstance()->getService('session')->get('ACCESS_PROFILE');
        $operator = $this->environment->getCurrentCdoperador();
        $idViewZmartDashboard = $this->environment->idViewZmartDashboard;

        $param = array(
            "requestType" => "Row",
            "row" => array(
                "dataRow" => array(
                    'drillDownFilters' => $drillDownFilters,
                    'viewId' => $idViewZmartDashboard
                ),
                "environment" => array("OPERATOR_CD" => $operator, "ACCESS_PROFILE" => $accessProfile)
            )
        );
        $data = InstanceProvider::getLibsUtilities()->callWebService($token, '/api/findbuildWidgetByViewId', $param, 'findbuildWidgetByViewId',
                Utilities::CLIENT_SECRET_LIBRARIES, $this->environment->zmartUrl, true);
        if(is_null($data)){
            throw new \Exception("Erro ao buscar metadados no Zmart.", 500);
        } else {
            return $data;
        }
    }

    /** Rotas para o zmart */
    public function findBiggerStructureInZmart()
    {
        $token = $this->environment->getTokenInZmart();
        $accessProfile = InstanceManager::getInstance()->getService('session')->get('ACCESS_PROFILE');
        $operator = $this->environment->getCurrentCdoperador();
        $idViewZmartDashboard = $this->environment->idViewZmartDashboard;
        $zmartStructureFilterId = $this->environment->zmartStructureFilterId;

        $param = array(
            "requestType" => "FilterData",
            "filter" => array(
                array('operator' => '=', 'value' => array('OPERATOR_CD' => $operator, 'ACCESS_PROFILE' => $accessProfile), 'name' => 'environment'),
                array('operator' => '=', 'value' => $idViewZmartDashboard, 'name' => 'VIEW_ID'),
                array('operator' => '=', 'value' => $zmartStructureFilterId, 'name' => 'zmartStructureFilterId')
            )
        );
        $data = InstanceProvider::getLibsUtilities()->callWebService($token, '/api/findBiggerStructure', $param, 'findBiggerStructure',
                Utilities::CLIENT_SECRET_LIBRARIES, $this->environment->zmartUrl, true);
        return $data;
    }

    /** Rotas para o zmart */
    public function fetchWidgetDataInZmart($filters)
    {
        $token = $this->environment->getTokenInZmart();
        $accessProfile = InstanceManager::getInstance()->getService('session')->get('ACCESS_PROFILE');
        $operator = $this->environment->getCurrentCdoperador();
        $idViewZmartDashboard = $this->environment->idViewZmartDashboard;
        $filters = Utilities::getValueInFilterDataSourceByName($filters, 'row');


        $param = array(
            "requestType" => "FilterData",
            "filter" => array(
                array('operator' => '=', 'value' => array('OPERATOR_CD' => $operator, 'ACCESS_PROFILE' => $accessProfile), 'name' => 'environment'),
                array(
                    'operator' => '=', 
                    'value' => array(
                        'drillDownFilters' => $filters['drillDownFilters'],
                        'structureId' => $filters['structureId'],
                        'period' => null,
                        'viewId' => $idViewZmartDashboard
                    ),
                    "name" => "value"
                )
            )
        );
        
        $data = InstanceProvider::getLibsUtilities()->callWebService($token, '/api/fetchWidgetData', $param, 'fetchWidgetData',
                Utilities::CLIENT_SECRET_LIBRARIES, $this->environment->zmartUrl, true);
        if(is_null($data)){
            throw new \Exception("Erro ao buscar metadados no Zmart.", 500);
        } else {
            return $data;
        }
    }

}