function ContainerMerger() {
	var self = this;

	var collections = {
		"fields": "id",
		"actions": "id",
		"events": "id",
		"fieldGroups": "id",
		"styles": "id",
		"dataSourceFilter": "id",
		"gaugeLevels": "id",
		"columns": "id"
	};

	var collectionsToExpand = {
		"widgets": "id"
	};

	var mergeCollectionItem = function(containerCollection, key, backupSuffix){
		return function(item) {
			for(var i in containerCollection){
				if(containerCollection[i][key] === item[key]){
					if (collections.hasOwnProperty(key)) {
						containerCollection[i] = item;
					} else {
						containerCollection[i] = self.merge(containerCollection[i], item, backupSuffix);
					}
					return;
				}
			}
			containerCollection.push(item);
		};
	};

	this.merge = function(container, containerPreference, backupSuffix){
		for (var key in containerPreference) {
			if (!collections.hasOwnProperty(key) && !collectionsToExpand.hasOwnProperty(key)) {
				if (backupSuffix) {
					container[key + "_" + backupSuffix] = container[key];
				}
				container[key] = containerPreference[key];
			} else {
				if(Util.isArray(containerPreference[key])) {
					container[key] = Util.parseToArray(container[key], true);
					var validationKey = collections.hasOwnProperty(key) ? collections[key] : collectionsToExpand[key];
					var mergeMethod = mergeCollectionItem(container[key], validationKey, backupSuffix);
					containerPreference[key].forEach(mergeMethod);
				}
			}
		}
		return container;
	};

	this.restore = function(container, backupSuffix) {
		for (var key in container) {
			if (!collections.hasOwnProperty(key) && !collectionsToExpand.hasOwnProperty(key)) {
				if (container.hasOwnProperty(key + "_" + backupSuffix)) {
					container[key] = container[key + "_" + backupSuffix];
					delete container[key + "_" + backupSuffix];
				}
			} else {
				if(Util.isArray(container[key])) {
					container[key] = Util.parseToArray(container[key], true);
					var validationKey = collections.hasOwnProperty(key) ? collections[key] : collectionsToExpand[key];
					var mergeMethod = makeRestoreCollectionMethod(container[key], validationKey, backupSuffix);
					container[key].forEach(mergeMethod);
				}
			}
		}
		return container;
	};

	function makeRestoreCollectionMethod(containerCollection, key, backupSuffix){
		return function(item) {
			for(var i in containerCollection){
				if(containerCollection[i][key] === item[key]){
					containerCollection[i] = self.restore(containerCollection[i], backupSuffix);
					return;
				}
			}
		};
	}

}